import clip

from . import BaseTest, Stream


class TestExamples(BaseTest):
	'''Test documentation examples to protect against regressions.

	If any of these tests fail, then the associated docs need to be updated.
	Similarly, if any of the examples are updated they should be reflected
	in the tests here.
	'''

	def test_readme(self):
		# Shopping list example in README and Getting Started section
		app, out, err = self.embed()

		@app.main(description='A very unhelpful shopping list CLI program')
		def shopping():
			pass

		@shopping.subcommand(description='Add an item to the list')
		@clip.arg('item', required=True)
		@clip.opt('-q', '--quantity', default=1, help='How many of the item to get')
		def add(item, quantity):
			clip.echo('Added "{} - {}" to the list'.format(item, quantity))

		@shopping.subcommand(description='See all items on the list')
		@clip.flag('--sorted', help='View items in alphabetical order')
		def view(sorted):
			clip.echo('This is your {}sorted list'.format('' if sorted else 'un'))

		inputs = [
			'-h',
			'add -h',
			'add',
			'add cookies -q 10',
			'view',
			'view --sorted'
		]
		for e in inputs:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, [
			'''shopping: A very unhelpful shopping list CLI program

Usage: shopping {{options}} {{subcommand}}

Options:
  -h, --help  Show this help message and exit

Subcommands:
  add   Add an item to the list
  view  See all items on the list
''',
			'''shopping add: Add an item to the list

Usage: add {{arguments}} {{options}}

Arguments:
  item [text]  

Options:
  -h, --help            Show this help message and exit
  -q, --quantity [int]  How many of the item to get (default: 1)
''',
			'Added "cookies - 10" to the list\n',
			'This is your unsorted list\n',
			'This is your sorted list\n'
		])
		self.assertEqual(err._writes, [
			'Error: Missing parameter "item".\n'
		])

	def test_getting_started(self):
		# Echo example
		app, out, _ = self.embed()

		@app.main()
		@clip.arg('words', nargs=-1)
		def echo(words):
			clip.echo(' '.join(words))

		app.run('Hello world!').run('idk     what i doinggggg      hahahaa a')
		self.assertEqual(out._writes, ['Hello world!\n', 'idk what i doinggggg hahahaa a\n'])

	def test_commands(self):
		# default
		app, out, _ = self.embed()

		@app.main(default='x --num 19')
		def f():
			pass

		@f.subcommand(default='-h')
		@clip.opt('--num', type=int, required=True)
		def x(num):
			clip.echo('I was invoked with the number {}!'.format(num))

		for e in ['x --num 5', 'x', '']:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, [
			'I was invoked with the number 5!\n',
			'''f x

Usage: x {{options}}

Options:
  -h, --help   Show this help message and exit
  --num [int]  
''',
			'I was invoked with the number 19!\n'
		])

		# description
		app, out, _ = self.embed()

		@app.main(description='This thing does awesome stuff!')
		def f():
			pass

		@f.subcommand(description='This is a sweet subcommand!')
		def sub():
			pass

		for e in ['-h', 'sub -h']:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, [
			'''f: This thing does awesome stuff!

Usage: f {{options}} {{subcommand}}

Options:
  -h, --help  Show this help message and exit

Subcommands:
  sub  This is a sweet subcommand!
''',
			'''f sub: This is a sweet subcommand!

Usage: sub {{options}}

Options:
  -h, --help  Show this help message and exit
'''
		])

		# epilogue
		app, out, _ = self.embed()

		@app.main(epilogue='So long and thanks for all the fish!')
		def f():
			pass

		try:
			app.run('-h')
		except clip.ClipExit:
			pass
		self.assertEqual(out._writes[0], '''f

Usage: f {{options}}

Options:
  -h, --help  Show this help message and exit

So long and thanks for all the fish!
''')

		# tree_view
		app, out, _ = self.embed()

		@app.main(tree_view='-t')
		@clip.flag('-t', '--tree', hidden=True)
		def w():
			pass

		@w.subcommand()
		def x():
			pass

		@x.subcommand()
		def y():
			pass

		@y.subcommand()
		def z():
			pass

		try:
			app.run('--tree')
		except clip.ClipExit:
			pass
		self.assertEqual(out._writes, ['w\n', '  x\n', '    y\n', '      z\n'])

	def test_parameters(self):
		# nargs
		app, out, err = self.embed()

		@app.main()
		@clip.arg('stuff', nargs=3)
		def f(stuff):
			clip.echo('You entered: {}'.format(stuff))

		for e in ['a', 'a b', 'a b c']:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, ["You entered: ['a', 'b', 'c']\n"])
		self.assertEqual(err._writes, ['Error: Not enough arguments for "stuff".\n'] * 2)

		# default
		app, out, _ = self.embed()

		@app.main()
		@clip.opt('--name', default='Joe')
		def f(name):
			clip.echo('Hello {}!'.format(name))

		app.run('').run('--name Dave')
		self.assertEqual(out._writes, ['Hello Joe!\n', 'Hello Dave!\n'])

		# type
		app, out, err = self.embed()

		@app.main()
		@clip.arg('numbers', nargs=-1, default=[1, 2, 3])
		def f(numbers):
			clip.echo(sum(numbers))

		for e in ['2 4 6 8 10', 'wuuutttt']:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, ['30\n'])
		self.assertEqual(err._writes, ['Error: Invalid type given to "numbers", expected int.\n'])

		# required
		app, out, err = self.embed()

		@app.main()
		@clip.flag('--needed', required=True)
		def f(needed):
			clip.echo('Ahh, I needed that.')

		for e in ['', '--needed']:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, ['Ahh, I needed that.\n'])
		self.assertEqual(err._writes, ['Error: Missing parameter "needed".\n'])

		# callback
		app, out, _ = self.embed()

		def shout(value):
			clip.echo(' '.join(value).upper())

		@app.main()
		@clip.arg('words', nargs=-1, callback=shout)
		def f(words):
			pass

		app.run('i feel da powah!')
		self.assertEqual(out._writes[0], 'I FEEL DA POWAH!\n')

		# help
		app, out, _ = self.embed()

		@app.main()
		@clip.flag('--panic', help='Don\'t do this')
		def f(panic):
			pass

		try:
			app.run('-h')
		except clip.ClipExit:
			pass
		self.assertEqual(out._writes[0], '''f

Usage: f {{options}}

Options:
  -h, --help  Show this help message and exit
  --panic     Don't do this
''')

	def test_inheriting_parameters(self):
		# Calculator (here we only test the final version for expected output)
		from functools import reduce

		app, out, _ = self.embed()

		@app.main()
		@clip.arg('numbers', nargs=-1, type=int, inherit_only=True)
		@clip.flag('-s', '--silent', inherit_only=True)
		def calculator():
			pass

		@calculator.subcommand(inherits=['numbers', '-s'])
		def add(numbers, silent):
			if not silent:
				clip.echo('Add these numbers? Okay, here we gooooo!')
			clip.echo(sum(numbers))

		@calculator.subcommand(inherits=['numbers', '--silent'])
		def multiply(numbers, silent):
			if not silent:
				clip.echo('Wa-hoo, let\'s-a multiply these numbers!')
			clip.echo(reduce(lambda x, y: x * y, numbers) if numbers else 0)

		for e in ['add 1 3 5 7', 'multiply 1 3 5 7', 'add -s 1 3 5 7', '-s add 1 3 5 7']:
			app.run(e)
		self.assertEqual(out._writes, [
			'Add these numbers? Okay, here we gooooo!\n', '16\n',
			'Wa-hoo, let\'s-a multiply these numbers!\n', '105\n',
			'16\n',
			'16\n'
		])

		# Going deeper
		app, out, _ = self.embed()

		@app.main()
		@clip.flag('-a')
		def w(a):
			clip.echo('a in w: {}'.format(a))

		@w.subcommand()
		@clip.flag('-b')
		def x(b):
			clip.echo('b in x: {}'.format(b))

		@x.subcommand(inherits=['a'])
		@clip.flag('-c')
		def y(a, c):
			clip.echo('a in y: {}'.format(a))
			clip.echo('c in y: {}'.format(c))

		@y.subcommand(inherits=['a', 'b', 'c'])
		@clip.flag('-d')
		def z(a, b, c, d):
			clip.echo('All together now: {}'.format((a, b, c, d)))

		app.run('-a x -b y -c z -d')
		self.assertEqual(out._writes, [
			'a in w: True\n',
			'b in x: True\n',
			'a in y: True\n',
			'c in y: True\n',
			'All together now: (True, True, True, True)\n'
		])

	def test_embedding(self):
		# Multiple apps (slightly modified for testing)
		out1 = Stream()
		out2 = Stream()
		app1 = clip.App(stdout=out1, name='app1')
		app2 = clip.App(stdout=out2, name='app2')
		clip.echo('App 1 via name', app='app1')
		clip.echo('App 2 via name', app='app2')
		app1.echo('App 1 directly')
		app2.echo('App 2 directly')
		clip.echo('Broadcast!')
		self.assertEqual(out1._writes, [
			'App 1 via name\n',
			'App 1 directly\n',
			'Broadcast!\n'
		])
		self.assertEqual(out2._writes, [
			'App 2 via name\n',
			'App 2 directly\n',
			'Broadcast!\n'
		])

	def test_extending_clip(self):
		# The sorting program
		class Choice(clip.Option):
			'''A special option that must be chosen from a list of valid values.

			The default value will be the first item of the list.
			'''

			def __init__(self, param_decls, **attrs):
				try:
					self._choices = attrs.pop('choices')
				except KeyError:
					raise AttributeError('You must specify the choices to select from')
				if not isinstance(self._choices, list) or len(self._choices) == 0:
					raise AttributeError('"choices" must be a nonempty list of valid values')
				attrs['nargs'] = 1
				attrs['default'] = self._choices[0]
				clip.Option.__init__(self, param_decls, **attrs)

			def consume(self, tokens):
				tokens.pop(0)  # Pop the choice from the tokens array
				selected = tokens.pop(0)
				if selected not in self._choices:
					clip.exit('Error: "{}" is not a valid choice (choose from {}).'.format(selected, ', '.join(self._choices)), True)
				clip.Parameter.post_consume(self, selected)
				return tokens

		def choice(*param_decls, **attrs):
			return clip._make_param(Choice, param_decls, **attrs)

		app, out, err = self.embed()

		@app.main()
		@choice('-t', '--type', name='sort_type', choices=['quicksort', 'bubblesort', 'mergesort'])
		def sort(sort_type):
			clip.echo('You selected {}'.format(sort_type))

		for e in ['', '-t spaghettisort', '-t mergesort']:
			try:
				app.run(e)
			except clip.ClipExit:
				pass
		self.assertEqual(out._writes, ['You selected quicksort\n', 'You selected mergesort\n'])
		self.assertEqual(err._writes, ['Error: "spaghettisort" is not a valid choice (choose from quicksort, bubblesort, mergesort).\n'])



import clip


class Choice(clip.Option):
	'''A special option that must be chosen from a list of valid values.

	The default value will be the first item of the list.
	'''

	def __init__(self, param_decls, **attrs):
		try:
			self._choices = attrs.pop('choices')
		except KeyError:
			raise AttributeError('You must specify the choices to select from')
		if not isinstance(self._choices, list) or len(self._choices) == 0:
			raise AttributeError('"choices" must be a nonempty list of valid values')
		attrs['nargs'] = 1
		attrs['default'] = self._choices[0]
		clip.Option.__init__(self, param_decls, **attrs)

	def consume(self, tokens):
		tokens.pop(0)  # Pop the choice from the tokens array
		selected = tokens.pop(0)
		if selected not in self._choices:
			clip.exit('Error: "{}" is not a valid choice (choose from {}).'.format(selected, ', '.join(self._choices)), True)
		clip.Parameter.post_consume(self, selected)
		return tokens


def choice(*param_decls, **attrs):
	return clip._make_param(Choice, param_decls, **attrs)


app = clip.App()

@app.main()
@choice('-t', '--type', name='sort_type', choices=['quicksort', 'bubblesort', 'mergesort'])
def sort(sort_type):
	clip.echo('You selected {}'.format(sort_type))

if __name__ == '__main__':
	try:
		app.run()
	except clip.ClipExit:
		pass



import clip
import sys


class Stream(object):
	def __init__(self, f):
		self._f = f

	def write(self, message):
		self._f(message)


def stream_a(message):
	sys.stdout.write('From stream A: {}'.format(message))

def stream_b(message):
	sys.stdout.write('From stream B: {}'.format(message))


app1 = clip.App(stdout=Stream(stream_a), name='app1')
app2 = clip.App(stdout=Stream(stream_b), name='app2')

clip.echo('App 1 via name', app='app1')
clip.echo('App 2 via name', app='app2')

app1.echo('App 1 directly')
app2.echo('App 2 directly')

clip.echo('Broadcast!')



import clip

app = clip.App()

@app.main()
@clip.flag('-s')
def inheritance(s):
	clip.echo('s from main: {}'.format(s))

@inheritance.subcommand(inherits=['-s'])
@clip.flag('-t')
def a(s, t):
	clip.echo('s from a: {}'.format(s))
	clip.echo('t from a: {}'.format(t))

@a.subcommand(inherits=['-s', '-t'])
def b(s, t):
	clip.echo('s from b: {}'.format(s))
	clip.echo('t from b: {}'.format(t))

if __name__ == '__main__':
	try:
		app.run()
	except clip.ClipExit:
		pass



# -*- coding: utf-8 -*-
import unittest
import contextlib

import clip


@contextlib.contextmanager
def mock_clip_input(s):
	cache = clip.input
	def step(_):
		return s.pop(0)
	clip.input = step
	yield
	clip.input = cache


class Stream(object):
	def __init__(self):
		self._writes = []

	def write(self, message):
		self._writes.append(message)


class BaseTest(unittest.TestCase):
	'''Base class for tests in this file.

	This should:
	  - Hold generic test apps and expected values, bound to self
	  - Expose generic methods useful to other tests
	'''

	def setUp(self):
		# Flush apps from previous tests, or they get written to again
		clip.clip_globals._streams = {}

	def make_kitchen_sink_app(self):
		app = clip.App()
		self.a = []
		self.b = []

		@app.main()
		@clip.flag('-a', '--apple')
		@clip.flag('--banana', '-b')
		@clip.opt('--file', name='filename')
		@clip.arg('donut')
		def a(apple, banana, filename, donut):
			self.a = [apple, banana, filename, donut]

		@a.subcommand()
		@clip.flag('-t')
		@clip.flag('--long-thing')
		@clip.arg('args', nargs=-1)
		def b(t, long_thing, args):
			self.b = [t, long_thing, args]

		return app

	def embed(self):
		out, err = Stream(), Stream()
		app = clip.App(stdout=out, stderr=err)
		return app, out, err

	def make_embedded_app(self):
		app, out, err = self.embed()

		@app.main()
		@clip.opt('--to-out')
		@clip.opt('--to-err')
		def f(to_out, to_err):
			if to_out is not None:
				clip.echo(to_out)
			if to_err is not None:
				clip.echo(to_err, err=True)

		return app, out, err


class TestGlobals(BaseTest):

	def test_clip_globals(self):
		out, err = Stream(), Stream()
		cg = clip.ClipGlobals()
		cg.add_streams(out, err)
		cg.echo('My my')
		cg.echo('What have I done?', err=True)
		for i in range(10):
			cg.echo('.', nl=False)
		self.assertEqual(out._writes, ['My my\n'] + ['.'] * 10)
		self.assertEqual(err._writes, ['What have I done?\n'])

	def test_echo(self):
		_, out, _ = self.embed()
		# Make sure Unicode works properly
		clip.echo('你好')
		self.assertEqual(out._writes, ['你好\n'])

	def test_exit(self):
		self.embed()
		# Standard case, custom message
		try:
			clip.exit('Woot!')
		except clip.ClipExit as e:
			self.assertEqual(e.message, 'Woot!')
		# Error condition
		try:
			clip.exit(err=True)
		except clip.ClipExit as e:
			self.assertTrue(e.message.startswith('clip exiting'))
			self.assertEqual(e.status, 1)

	def test_confirm(self):
		self.embed()
		# All the standard accepted entries
		with mock_clip_input(['y', 'n', 'Y', 'N', 'yEs', 'No', 'YES', 'no']):
			for e in [True, False, True, False, True, False, True, False]:
				self.assertEqual(clip.confirm('?'), e)
		# Abort
		with mock_clip_input(['n']):
			with self.assertRaises(clip.ClipExit):
				clip.confirm('?', abort=True)
		# Mistaken entries
		_, out, _ = self.embed()
		with mock_clip_input(['', 'boop', 'PAHHHHHH', 'NO']):
			self.assertFalse(clip.confirm('?'))
		self.assertEqual(len(out._writes), 3)

	def test_prompt(self):
		# Standard with confirm
		with mock_clip_input(['7', '7']):
			self.assertEqual(clip.prompt('?', type=int, confirm=True), 7)
		# Bad input
		_, out, _ = self.embed()
		with mock_clip_input(['', 'hehehe', '42']):
			self.assertEqual(clip.prompt('?', type=int), 42)
		self.assertEqual(len(out._writes), 2)
		# Interrupt
		def cause_interrupt():
			raise KeyboardInterrupt
		with mock_clip_input(['']):
			with self.assertRaises(clip.ClipExit):
				clip.prompt('?', default=cause_interrupt)
		# Skip
		with mock_clip_input(['', '']):
			self.assertIsNone(clip.prompt('?', skip=True))
			self.assertIsNone(clip.prompt('?', confirm=True, skip=True))


class TestParse(BaseTest):

	def test_kitchen_sink(self):
		expected = {
			'apple': True,
			'banana': True,
			'filename': 'pie.txt',
			'donut': 'chocolate',
			'b': {
				't': False,
				'long_thing': True,
				'args': ['yum', 'yo']
			}
		}
		actuals = [
			'-ab --file pie.txt chocolate b --long-thing yum yo',
			'chocolate -a --file pie.txt --banana b --long-thing yum yo'
		]
		for actual in actuals:
			self.assertEqual(self.make_kitchen_sink_app().parse(actual.split()), expected)

	def test_required(self):
		app, _, err = self.embed()

		@app.main()
		@clip.opt('-o')
		@clip.flag('-r', required=True)
		def f(o, r):
			pass

		with self.assertRaises(clip.ClipExit):
			app.run('-o joe')
		self.assertTrue('Missing' in err._writes[0])

	def test_common_errors(self):
		app, _, err = self.embed()

		@app.main()
		def f():
			pass

		@f.subcommand()
		@clip.opt('--tokens', nargs=4)
		def nargs(tokens):
			pass

		@f.subcommand()
		@clip.opt('--int', type=int)
		@clip.arg('ints', type=int, nargs=3)
		def coerce(int, ints):
			pass

		for e in ['nargs --tokens 1 2 3', 'coerce --int pie', 'coerce this aint right']:
			with self.assertRaises(clip.ClipExit):
				app.run(e)
		for i, e in enumerate(['Not enough', 'Invalid type', 'Invalid type']):
			self.assertTrue(e in err._writes[i])


class TestInvoke(BaseTest):

	def test_invoke(self):
		self.make_kitchen_sink_app().run('-ab --file pie.txt chocolate b --long-thing yum yo')
		self.assertEqual(self.a, [True, True, 'pie.txt', 'chocolate'])
		self.assertEqual(self.b, [False, True, ['yum', 'yo']])

	def test_reset(self):
		app = self.make_kitchen_sink_app()
		app.run('b a n a n a')
		self.assertEqual(self.b[2], 'a n a n a'.split())
		# If reset works, we should be able to run another command right away
		app.run('b o o p')
		self.assertEqual(self.b[2], 'o o p'.split())

	def test_run(self):
		app, out, err = self.make_embedded_app()
		app.run(['--to-out', 'list']).run('--to-out string').run('--to-err "two words"')
		self.assertEqual(out._writes, ['list\n', 'string\n'])
		self.assertEqual(err._writes, ['two words\n'])

	def test_version(self):
		app, out, _ = self.embed()

		def print_version(value):
			clip.exit('Version 0.0.0')

		@app.main()
		@clip.flag('--version', callback=print_version, hidden=True)
		def f():
			clip.echo('Should not be called')

		with self.assertRaises(clip.ClipExit):
			app.run('--version')
		self.assertEqual(out._writes, ['Version 0.0.0\n'])

	def test_call_command(self):
		app, out, _ = self.embed()

		@app.main()
		@clip.arg('number', type=int)
		def f(number):
			clip.echo('Hey there {}'.format(number))
			return 42 * number

		self.assertEqual(f(5), 210)
		self.assertEqual(out._writes[0], 'Hey there 5\n')

	def test_default_is_function(self):
		app, out, _ = self.embed()

		def fn():
			clip.echo('No name provided')

		@app.main()
		@clip.opt('--name', default=fn)
		def f(name):
			if name:
				clip.echo('Hello {}!'.format(name))

		app.run('').run('--name Dave')
		self.assertEqual(out._writes, ['No name provided\n', 'Hello Dave!\n'])


class TestCommand(BaseTest):

	def test_default(self):
		app, out, _ = self.embed()

		@app.main(default='x')
		def f():
			pass

		@f.subcommand()
		def x():
			clip.echo('x invoked!')

		app.run('x').run('')
		self.assertEqual(out._writes, ['x invoked!\n', 'x invoked!\n'])


class TestHelp(BaseTest):

	def test_basic_help(self):
		app, out, _ = self.embed()

		@app.main()
		def f():
			pass

		try:
			app.run('-h')
		except clip.ClipExit:
			pass
		self.assertEqual(out._writes[0], '''f

Usage: f {{options}}

Options:
  -h, --help  Show this help message and exit
''')

	def test_full_help(self):
		app, out, _ = self.embed()

		@app.main(description='Description', epilogue='So long and thanks for all the fish!')
		@clip.arg('yo')
		@clip.opt('-s', '--something', type=int, nargs=3, default=[1, 2, 3], help='Does stuff')
		@clip.flag('-n', '--nothing')
		def f(yo, something, nothing):
			pass

		@f.subcommand(description='Oh lookee! A subcommand!')
		def x():
			pass

		try:
			app.run('-h')
		except clip.ClipExit:
			pass
		self.assertEqual(out._writes[0], '''f: Description

Usage: f {{arguments}} {{options}} {{subcommand}}

Arguments:
  yo [text]  

Options:
  -h, --help                Show this help message and exit
  -s, --something [int...]  Does stuff (default: [1, 2, 3])
  -n, --nothing             

Subcommands:
  x  Oh lookee! A subcommand!

So long and thanks for all the fish!
''')


class TestInheritance(BaseTest):

	def test_inheritance(self):
		app, out, err = self.embed()

		@app.main()
		@clip.flag('--flag')
		def f(flag):
			clip.echo(flag)

		@f.subcommand()
		def x():
			pass

		@x.subcommand(inherits=['flag'])
		def y(flag):
			clip.echo(flag, err=True)

		app.run('--flag x y')
		self.assertEqual(out._writes, ['True\n'])
		self.assertEqual(err._writes, ['True\n'])

	def test_inherits_args(self):
		app, out, _ = self.embed()

		@app.main()
		@clip.flag('-b', '--boop', inherit_only=True)
		def f():
			pass

		@f.subcommand(inherits=['boop'])
		def by_name(boop):
			clip.echo("name " + str(boop))

		@f.subcommand(inherits=['-b'])
		def by_short(boop):
			clip.echo("short " + str(boop))

		@f.subcommand(inherits=['--boop'])
		def by_long(boop):
			clip.echo("long " + str(boop))

		app.run('-b by_name').run('-b by_short').run('-b by_long')
		self.assertEqual(out._writes, ['name True\n', 'short True\n', 'long True\n'])

	def test_inherit_only(self):
		app, out, err = self.embed()

		@app.main()
		@clip.flag('-s', inherit_only=True)
		@clip.flag('-t')
		def f(t):
			clip.echo(t)

		@f.subcommand(inherits=['-s', '-t'])
		def sub(s, t):
			clip.echo(s, err=True)
			clip.echo(t, err=True)

		parsed = app.parse(['-st', 'sub'])
		self.assertEqual(parsed, {
			't': True,
			'sub': {
				's': True,
				't': True
			}
		})
		app.invoke(parsed)
		self.assertEqual(out._writes, ['True\n'])
		self.assertEqual(err._writes, ['True\n', 'True\n'])


class TestEmbedding(BaseTest):

	def test_streams(self):
		app, out, err = self.make_embedded_app()
		app.run('--to-out out1 --to-err err1')
		self.assertEqual(out._writes, ['out1\n'])
		self.assertEqual(err._writes, ['err1\n'])

	def test_exit_message(self):
		# Exiting should print a message
		_, out, _ = self.make_embedded_app()
		with self.assertRaises(clip.ClipExit):
			clip.exit('Exiting!')
		self.assertEqual(out._writes, ['Exiting!\n'])

	def test_embedded_confirm(self):
		self.cache = None
		def custom_input(prompt):
			self.cache = prompt
			return 'y'
		self.assertTrue(clip.confirm('?', default='no', input_function=custom_input))
		self.assertEqual(self.cache, '? [y/N]: ')

	def test_embedded_prompt(self):
		self.cache = None
		def custom_input(prompt):
			self.cache = prompt
			return 37
		self.assertEqual(clip.prompt('?', default=42, input_function=custom_input), 37)
		self.assertEqual(self.cache, '? [42]: ')


class TestMistakes(BaseTest):
	'''These are mistakes a programmer would make using clip.
	'''

	def test_app_mistakes(self):
		# App should not define more than one main command
		app = self.make_kitchen_sink_app()
		with self.assertRaises(AttributeError):

			@app.main()
			def f():
				pass

		# App should define at least one main command
		with self.assertRaises(AttributeError):
			clip.App().run('something')

	def test_command_mistakes(self):
		# Giving a command a callback that is already a command
		with self.assertRaises(TypeError):
			app = clip.App()

			@app.main()
			def f():
				pass

			@f.subcommand()
			@f.subcommand()
			def sub():
				pass

		# Giving main an "inherits"
		with self.assertRaises(AttributeError):
			app = clip.App()

			@app.main(inherits=['whoops'])
			def f(whoops):
				pass

		# Inheriting a parameter that doesn't exist
		with self.assertRaises(AttributeError):
			app = clip.App()

			@app.main()
			def f():
				pass

			@f.subcommand(inherits=['whoops'])
			def sub(whoops):
				pass

		# Giving tree_view something other than a flag
		with self.assertRaises(TypeError):
			app = clip.App()

			@app.main(tree_view='arg')
			@clip.arg('arg', help='whoops')
			def f(arg):
				pass

	def test_parameter_mistakes(self):
		# If nargs != 1 and default is not a list
		with self.assertRaises(TypeError):
			app = clip.App()

			@app.main()
			@clip.opt('-a', nargs=-1, default='whoops')
			def f(a):
				pass

	def test_argument_mistakes(self):
		# Specifying more than one name for an argument
		with self.assertRaises(TypeError):
			app = clip.App()

			@app.main()
			@clip.arg('name', 'whoops')
			def f(name):
				pass



import clip
from functools import reduce

app = clip.App()

@app.main()
@clip.arg('numbers', nargs=-1, type=int, inherit_only=True)
@clip.flag('-s', '--silent', inherit_only=True)
def calculator():
	pass

@calculator.subcommand(inherits=['numbers', '-s'])
def add(numbers, silent):
	if not silent:
		clip.echo('Add these numbers? Okay, here we gooooo!')
	clip.echo(sum(numbers))

@calculator.subcommand(inherits=['numbers', '--silent'])
def multiply(numbers, silent):
	if not silent:
		clip.echo('Wa-hoo, let\'s-a multiply these numbers!')
	clip.echo(reduce(lambda x, y: x * y, numbers) if numbers else 0)

if __name__ == '__main__':
	try:
		app.run()
	except clip.ClipExit:
		pass



'''
clip.py: Embeddable, composable [c]ommand [l]ine [i]nterface [p]arsing

Copyright: (c) 2015 William Gaul
License: MIT, see LICENSE for more details
'''
import itertools
import shlex
import sys
import uuid


########################################
# COMPATIBILITY & UTILITIES
########################################

PY2 = sys.version_info[0] == 2

input = raw_input if PY2 else input
text_type = basestring if PY2 else str
to_str = lambda s: '{}'.format(s)
def is_func(e):
	return hasattr(e, '__call__')
def iteritems(d):
	return d.iteritems() if PY2 else d.items()

def get_input_fn(f=None, invisible=False):
	if f is not None:
		return f
	if not invisible:
		return input
	import getpass
	return getpass.getpass

def prompt_fn(f, s, default=None, type=None, skip=False, repeat=False):
	default = default or ''
	while True:
		try:
			ret = f(s) or (default() if is_func(default) else default)
			if skip and not ret:
				return None
			ret = type(ret) if type is not None else ret
		except (KeyboardInterrupt, EOFError):
			raise_abort()
		except ValueError:
			echo('Please provide an {}'.format(type.__name__))
		else:
			if ret or not repeat:
				return ret

def determine_type(t, default):
	if t is None:
		if default is not None:
			if isinstance(default, list):
				return type(default[0]) if len(default) > 0 else None
			return None if is_func(default) else type(default)
		return None
	return t


########################################
# GLOBAL CLASSES/METHODS
########################################

class ClipGlobals(object):

	def __init__(self):
		self._streams = {}

	def _write(self, message, stream, nl=True):
		stream.write(to_str(message) + ("\n" if nl else ""))
		# Custom streams may not implement a flush() method
		if hasattr(stream, 'flush'):
			stream.flush()

	def _broadcast(self, message, err=False, nl=True):
		for k, v in iteritems(self._streams):
			self._write(message, v['err' if err else 'out'], nl)

	def echo(self, message, err=False, nl=True, app=None):
		if not self._streams:
			raise AttributeError('No streams have been initialized')
		if app is None:
			self._broadcast(message, err, nl)
		else:
			self._write(message, self._streams[app]['err' if err else 'out'], nl)

	def add_streams(self, out, err, app=None):
		self._streams[app] = {
			'out': out or sys.stdout,
			'err': err or sys.stderr
		}


class ClipExit(Exception):
	def __init__(self, message=None, status=0):
		self.message = message or 'clip exiting with status {}'.format(status)
		self.status = status
	def __str__(self):
		return repr(self.message)


clip_globals = ClipGlobals()

def echo(message, err=False, nl=True, app=None):
	clip_globals.echo(message, err, nl, app)

def exit(message=None, err=False, app=None):
	if message:
		echo(message, err, app=app)
	raise ClipExit(message, 1 if err else 0)

def raise_abort():
	exit('Operation aborted by user', True)

def confirm(prompt, default=None, show_default=True, abort=False, input_function=None):
	'''Prompts for confirmation from the user.
	'''
	valid = {
		'yes': True,
		'y': True,
		'no': False,
		'n': False
	}
	input_function = get_input_fn(input_function)
	if default not in ['yes', 'no', None]:
		default = None
	if show_default:
		prompt = '{} [{}/{}]: '.format(prompt,
				'Y' if default == 'yes' else 'y',
				'N' if default == 'no' else 'n')
	while True:
		choice = prompt_fn(input_function, prompt, default).lower()
		if choice in valid:
			if valid[choice] == False and abort:
				raise_abort()
			return valid[choice]
		else:
			echo('Please respond with "yes" or "no" (or "y" or "n").')

def prompt(text, default=None, show_default=True, invisible=False,
           confirm=False, skip=False, type=None, input_function=None):
	'''Prompts for input from the user.
	'''
	t = determine_type(type, default)
	input_function = get_input_fn(input_function, invisible)
	if default is not None and show_default:
		text = '{} [{}]: '.format(text, default)
	while True:
		val = prompt_fn(input_function, text, default, t, skip, repeat=True)
		if not confirm or (skip and val is None):
			return val
		if val == prompt_fn(input_function, 'Confirm: ', default, t, repeat=True):
			return val
		echo('Error: The two values you entered do not match', True)


########################################
# PARAMETER METHODS
########################################

def _memoize_param(f, param):
	if isinstance(f, Command):
		f._params.add(param)
	else:
		if not hasattr(f, '__clip_params__'):
			f.__clip_params__ = []
		f.__clip_params__.append(param)

def _make_param(cls, param_decls, **attrs):
	def decorator(f):
		_memoize_param(f, cls(param_decls, **attrs))
		return f
	return decorator

def arg(*param_decls, **attrs):
	return _make_param(Argument, param_decls, **attrs)

def opt(*param_decls, **attrs):
	return _make_param(Option, param_decls, **attrs)

def flag(*param_decls, **attrs):
	return _make_param(Flag, param_decls, **attrs)


########################################
# PARAMETER CLASSES
########################################

class Parameter(object):
	'''Base class for all parameters associated with a command.

	A parameter consists of several *parameter declarations* followed by
	keyword *attributes*. The parameter declarations represent the parameter
	as it would be entered by users, e.g. 'arg' or ('-o', '--opt').
	'''

	def __init__(self, param_decls, name=None, nargs=1, default=None,
	             type=None, required=False, callback=None, hidden=False,
	             inherit_only=False, help=None):
		self._decls = param_decls
		self._name = name or self._make_name(param_decls)
		self._nargs = nargs
		self._default = self._make_default(default, nargs)
		self._type = determine_type(type, self._default)
		self._required = required
		self._callback = callback
		self._hidden = hidden
		self._inherit_only = inherit_only
		self._help = help

		self.reset()  # Do an initial reset to prime the parameter

	def reset(self):
		self._value = None  # The parsed value of this parameter
		self._satisfied = False  # True when this parameter has consumed tokens

	def _make_name(self, decls):
		raise NotImplementedError('_make_name/1 must be implemented by child classes')

	def _make_default(self, default, nargs):
		# The default is a function, just assume it returns the correct type
		if is_func(default):
			return default
		if nargs != 0 and nargs != 1:
			default = default or []
			if not isinstance(default, list):
				raise TypeError('Default value must be list with nargs={}'.format(nargs))
		return default

	def _get_help_name(self):
		return self._name

	def _get_help(self):
		name = [self._get_help_name()]
		if self._nargs != 0:
			name.append('[{}{}]'.format(
					self._type.__name__ if self._type else 'text',
					'' if self._nargs == 1 else '...'))
		desc = []
		if self._help is not None:
			desc.append(self._help)
		if self._default:
			desc.append('(default: {})'.format(self._default))
		return [' '.join(name), ' '.join(desc)]

	def name(self):
		return self._name

	def hidden(self):
		return self._hidden

	def inherit_only(self):
		return self._inherit_only

	def value(self):
		return self._value

	def satisfied(self):
		return self._satisfied

	def consume(self, tokens):
		'''Have this parameter consume some tokens.

		This stores the consumed value for later use and returns the
		modified tokens array for further processing.
		'''
		n = len(tokens) if self._nargs == -1 else self._nargs
		if n > len(tokens):
			exit('Error: Not enough arguments for "{}".'.format(self._name), True)
		try:
			consumed = [self._type(e) if self._type is not None else e for e in tokens[:n]]
		except ValueError as e:
			exit('Error: Invalid type given to "{}", expected {}.'.format(
					self._name, self._type.__name__), True)
		if n == 1 and self._nargs == 1:
			consumed = consumed[0]
		self.post_consume(consumed)
		return tokens[n:]

	def post_consume(self, consumed):
		self._value = consumed
		self._satisfied = True
		# Parameter has been matched, so invoke the callback if any
		if self._callback is not None:
			self._callback(self._value)

	def set_default(self):
		# If we're calling this method, then this parameter wasn't provided
		if self._required:
			exit('Error: Missing parameter "{}".'.format(self._name), True)
		# The provided default can be a function, whose return value will be used
		self._value = self._default() if is_func(self._default) else self._default

	def matches(self, token):
		return not self._satisfied


class Argument(Parameter):
	'''A positional parameter.
	'''

	def _make_name(self, decls):
		if not len(decls) == 1:
			raise TypeError('Arguments take exactly 1 parameter declaration, got {}'.format(len(decls)))
		return decls[0]


class Option(Parameter):
	'''A (usually) optional parameter.
	'''

	def __init__(self, param_decls, **attrs):
		Parameter.__init__(self, param_decls, **attrs)

	def _make_name(self, decls):
		longest = sorted(list(decls), key=lambda x: len(x))[-1]
		return longest[2:].replace('-', '_').lower() if len(longest) > 2 else longest[1:]

	def _get_help_name(self):
		return ', '.join(self._decls)

	def consume(self, tokens):
		tokens.pop(0)  # Pop the opt from the tokens array
		return Parameter.consume(self, tokens)


class Flag(Option):
	'''A special option that consumes nothing and is True only if it appears.
	'''

	def __init__(self, param_decls, **attrs):
		attrs['nargs'] = 0
		attrs['default'] = False
		Option.__init__(self, param_decls, **attrs)

	def consume(self, tokens):
		tokens.pop(0)  # Pop the flag from the tokens array
		Parameter.post_consume(self, True)
		return tokens


class ParameterDict(object):

	def __init__(self, params):
		self._args = []
		self._opts = []
		self._args_map = {}
		self._opts_map = {}
		for param in params:
			self.add(param)

	def __contains__(self, key):
		return key in self._args_map or key in self._opts_map

	def __getitem__(self, key):
		if key in self._args_map:
			return self._args[self._args_map[key]]
		return self._opts[self._opts_map[key]]

	def add(self, param):
		l, m = (self._args, self._args_map) if isinstance(param, Argument) else (self._opts, self._opts_map)
		i = len(l)
		l.append(param)
		for decl in param._decls:
			m[decl] = i
		m[param.name()] = i

	def match(self, token):
		match = None
		if token in self._opts_map:
			possible = self._opts[self._opts_map[token]]
			if possible.matches(token):
				match = possible
		if match is None:
			for arg in self._args:
				if arg.matches(token):
					match = arg
					break
		return match

	def unsatisfied(self):
		return [p for p in self.all() if not p.satisfied()]

	def all(self):
		return self._args + self._opts

	def arguments(self):
		return self._args

	def options(self):
		return self._opts


########################################
# COMMAND METHODS
########################################

def _make_command(f, name, attrs):
	if isinstance(f, Command):
		raise TypeError('Callback is already a Command')
	try:
		params = f.__clip_params__
		params.reverse()
		del f.__clip_params__
	except AttributeError:
		params = []
	return Command(name=name or f.__name__.lower(), callback=f, params=params, **attrs)

def command(name=None, **attrs):
	def decorator(f):
		return _make_command(f, name, attrs)
	return decorator


########################################
# COMMAND CLASS
########################################

class Command(object):

	def __init__(self, name, callback, params, parent=None, default=None,
	             description=None, epilogue=None, inherits=None, tree_view=None):
		self._name = name
		self._callback = callback
		self._parent = parent
		self._default = default
		self._description = description
		self._epilogue = epilogue

		self._inherited = []
		# Add help to every command and shim in inherited parameters
		params.insert(0, Flag(('-h', '--help'), callback=self.help, hidden=True,
		                      help='Show this help message and exit'))
		if inherits is not None:
			if self._parent is None:
				raise AttributeError('A main function cannot inherit parameters')
			for e in inherits:
				param = self._parent._get_inherited_param(e)
				params.append(param)
				self._inherited.append(param.name())
		self._params = ParameterDict(params)
		# Handle tree view
		if tree_view is not None:
			c = self._params[tree_view]
			if not isinstance(c, Flag):
				raise TypeError('tree_view must be a Flag')
			c._callback = self.tree_view

		self._subcommands = {}

	def reset(self):
		# Reset all parameters associated with this command
		for param in self._params.all():
			param.reset()
		# Recurse into subcommands
		for v in self._subcommands.values():
			v.reset()

	def __call__(self, *args, **kwargs):
		return self._callback(*args, **kwargs)

	def _get_help(self):
		return [self._name, self._description or '']

	def _get_path(self):
		path = self._parent._get_path() if self._parent is not None else []
		return path + [self._name]

	def _get_inherited_param(self, name):
		if name in self._params:
			return self._params[name]
		if self._parent is not None:
			return self._parent._get_inherited_param(name)
		raise AttributeError('Unable to inherit parameter "{}"'.format(name))

	def name(self):
		return self._name

	def subcommand(self, name=None, **attrs):
		def decorator(f):
			attrs['parent'] = self
			cmd = command(name, **attrs)(f)
			self._subcommands[cmd._name] = cmd
			return cmd
		return decorator

	def parse(self, tokens):
		parsed = {}

		if not tokens and self._default is not None:
			tokens = self._default.split()

		# Pass 1: Forward - fill out parameter values based on input string
		while tokens:
			token = tokens[0]
			if token in self._subcommands:
				tokens.pop(0)
				parsed[token] = self._subcommands[token].parse(tokens)
				break  # The subcommand handles the remaining tokens
			match = self._params.match(token)
			if not match:
				exit('Error: Could not understand "{}".'.format(token), True)
			tokens = match.consume(tokens)

		# Pass 2: Backward - fill out missing parameters
		for param in self._params.unsatisfied():
			param.set_default()

		# Pass 3: Build the JSON-serializable object to return
		for param in self._params.all():
			if not param.hidden() and (param.name() in self._inherited or not param.inherit_only()):
				parsed[param.name()] = param.value()

		return parsed

	def invoke(self, parsed):
		# First invoke this command's callback
		self._callback(**{k: v for k, v in iteritems(parsed) if k not in self._subcommands})
		# Invoke subcommands (realistically only one should be invoked)
		for k, v in iteritems(parsed):
			if k in self._subcommands:
				self._subcommands[k].invoke(v)

	def help(self, value):
		help_parts = []
		usage = []

		# Header
		header = ' '.join(self._get_path())
		if self._description is not None:
			header = '{}: {}'.format(header, self._description)
		help_parts.append(header)

		# Main help sections (title, followed by 2-column list)
		def make_help_section(l, title):
			data = [e._get_help() for e in l]
			width = max(len(e[0]) for e in data) + 2
			return '\n'.join([title] + ['  {}{}'.format(e[0].ljust(width), e[1]) for e in data])

		args = self._params.arguments()
		if args:
			usage.append('{{arguments}}')
			help_parts.append(make_help_section(args, 'Arguments:'))
		opts = self._params.options()
		if opts:
			usage.append('{{options}}')
			help_parts.append(make_help_section(opts, 'Options:'))
		subs = sorted(self._subcommands.values(), key=lambda e: e.name())
		if subs:
			usage.append('{{subcommand}}')
			help_parts.append(make_help_section(subs, 'Subcommands:'))

		# Now we know the usage string, so insert it
		help_parts.insert(1, 'Usage: {} {}'.format(self._name, ' '.join(usage)))

		# Epilogue
		if self._epilogue is not None:
			help_parts.append(self._epilogue)

		exit('\n\n'.join(help_parts))

	def tree_view(self, value):
		echo('{}{}'.format(" " * (value - 1), self._name))
		subs = sorted(self._subcommands.values(), key=lambda e: e.name())
		if subs:
			for sub in subs:
				sub.tree_view(value + 2)
		if value == 1:
			exit()


########################################
# APP CLASS
########################################

class App(object):

	def __init__(self, stdout=None, stderr=None, name=None):
		self._main = None
		self._name = name or str(uuid.uuid4())
		clip_globals.add_streams(stdout, stderr, self._name)

	def _ping_main(self):
		if self._main is None:
			raise AttributeError('A main function must be assigned to this app')

	def main(self, name=None, **attrs):
		def decorator(f):
			if self._main is not None:
				raise AttributeError('A main function has already been assigned to this app')
			cmd = command(name, **attrs)(f)
			self._main = cmd
			return cmd
		return decorator

	def echo(self, message, err=False, nl=True):
		echo(message, err, nl, app=self._name)

	def exit(self, message=None, err=False):
		exit(message, err, app=self._name)

	def parse(self, tokens):
		'''Parses a list of tokens into a JSON-serializable object.

		The parsing proceeds from left to right and is greedy.

		Precedence order:
		  1. Parameters with active context. For example, an Option with
		     nargs=-1 will gobble all the remaining tokens.
		  2. Subcommands.
		  3. Parameters.

		The keys of the returned object are the names of parameters or
		subcommands. Subcommands are encoded as nested objects. Multiple
		parameters are encoded as lists. All other values are encoded as
		parameter-specified data types, or strings if not specified.
		'''
		self._ping_main()

		# Pre-parsing:
		#   1. Expand globbed options: -abc --> -a -b -c
		def is_globbed(s):
			return len(s) > 2 and s.startswith('-') and not s.startswith('--')
		expanded = [["-" + c for c in list(token[1:])] if is_globbed(token) else [token] for token in tokens]

		# Parsing: pass off to main command after flattening expanded tokens list
		return self._main.parse(list(itertools.chain.from_iterable(expanded)))

	def invoke(self, parsed):
		'''Invokes the app, given a parsed token object.
		'''
		self._ping_main()
		self._main.invoke(parsed)

	def reset(self):
		'''Returns the app to its initial state.

		This is necessary because parsing/invoking causes state to be stored
		in the commands and parameters, meaning that the app cannot be run
		again. After a reset, the app is freely available for reuse.
		'''
		self._ping_main()
		self._main.reset()

	def run(self, tokens=None):
		if tokens is None:
			tokens = sys.argv[1:]
		if isinstance(tokens, text_type):
			tokens = shlex.split(tokens)
		try:
			self.invoke(self.parse(tokens))
		finally:
			self.reset()  # Clean up so the app can be used again
		return self



try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup

setup(
	name='clip.py',
	version='0.3.1',
	url='https://github.com/willyg302/clip.py',
	license='MIT',
	author='William Gaul',
	author_email='willyg302@gmail.com',
	description='Embeddable, composable command line interface parsing',
	py_modules=['clip'],
	include_package_data=True,
	test_suite='tests',
	platforms='any',
	classifiers=[
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 3.4',
	],
)



import clip

from tornado import ioloop, web
from sockjs.tornado import SockJSConnection, SockJSRouter


class Stream(object):
	def __init__(self, f):
		self._f = f

	def write(self, message):
		self._f(message)


class Todo(object):
	def __init__(self, item):
		self._item = item
		self._completed = False

	def complete(self):
		self._completed = True

	def get_str(self):
		return '<span class="completed">{}</span>'.format(self._item) if self._completed else self._item


class IndexHandler(web.RequestHandler):
	def get(self):
		self.render('index.html')


class TodoConnection(SockJSConnection):

	def on_open(self, info):
		self._todos = []
		self._app = clip.App(stdout=Stream(self.on_out), stderr=Stream(self.on_err))

		@self._app.main()
		def todo():
			pass

		@todo.subcommand()
		@clip.arg('desc', nargs=-1, required=True)
		def add(desc):
			desc = ' '.join(desc)
			self._todos.append(Todo(desc))
			clip.echo('Added "{}" to the list of todos'.format(desc))

		@todo.subcommand()
		@clip.arg('index', type=int, required=True)
		def remove(index):
			try:
				removed = self._todos.pop(index - 1)
				clip.echo('Removed todo "{}"'.format(removed._item))
			except IndexError:
				clip.exit('Invalid todo index given', True)

		@todo.subcommand()
		@clip.arg('index', type=int, required=True)
		def complete(index):
			try:
				completed = self._todos[index - 1]
				completed.complete()
				clip.echo('Marked todo "{}" as completed'.format(completed._item))
			except IndexError:
				clip.exit('Invalid todo index given', True)

		@todo.subcommand(name='list')
		@clip.flag('-a', '--active')
		def list_todos(active):
			l = [e for e in self._todos if not e._completed] if active else self._todos
			clip.echo('\n'.join(['{}. {}'.format(i + 1, e.get_str()) for i, e in enumerate(l)]))

	def on_message(self, message):
		try:
			self._app.run(message)
		except clip.ClipExit:
			pass

	def on_out(self, message):
		self.send(message)

	def on_err(self, message):
		self.send('<span class="error">{}</span>'.format(message))


if __name__ == '__main__':
	TodoRouter = SockJSRouter(TodoConnection, '/todo')
	web.Application(
		[(r'/', IndexHandler)] + TodoRouter.urls
	).listen(8080)
	ioloop.IOLoop.instance().start()



import clip

app = clip.App()

@app.main(description='A very unhelpful shopping list CLI program')
def shopping():
	pass

@shopping.subcommand(description='Add an item to the list')
@clip.arg('item', required=True)
@clip.opt('-q', '--quantity', default=1, help='How many of the item to get')
def add(item, quantity):
	clip.echo('Added "{} - {}" to the list'.format(item, quantity))

@shopping.subcommand(description='See all items on the list')
@clip.flag('--sorted', help='View items in alphabetical order')
def view(sorted):
	clip.echo('This is your {}sorted list'.format('' if sorted else 'un'))

if __name__ == '__main__':
	try:
		app.run()
	except clip.ClipExit:
		pass
